#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Script to plot the distributions of all variables from a CSV file. 

This script is inteteded for VERY simples use cases. For anything more complex and customizable, 
prefear to use directly the function plot_dataframe_profile from the magikeda.univar package.


usage: plot_data_frame_profile.py [-h] [--sep SEP] [--encoding ENCODING]
                                  input_file

positional arguments:
  input_file           CSV file.

optional arguments:
  -h, --help           show this help message and exit
  --sep SEP            Separator used in CSV. Default is coma ",".
  --encoding ENCODING  File encoding used by the CSV file. Accepts any valid
                       encoding to pass to pandas.read_csv method. Default is
                       "latin1".
Erro: too few arguments
To exit: use 'exit', 'quit', or Ctrl-D.
An exception has occurred, use %tb to see the full traceback.

"""

import argparse
import sys
import pandas as pd

from magikeda import univar

class MyParser(argparse.ArgumentParser):
    """
        Configura argumentos da linha de comando.
    """
  
    def __init__(self):      
        """
          Configure command line args
        """
        super(MyParser, self).__init__()
        self.add_argument('input_file', help=u'CSV file.')        
        self.add_argument('--sep', default=',', help=u'Separator used in CSV. Default is coma ",".') 
        self.add_argument('--encoding', default='latin1', help=u'File encoding used by the CSV file. Accepts any valid encoding to pass to pandas.read_csv method. Default is "latin1". ') 
  
    def error(self, msg):
        """
            Mostra mensagens de erro quando argumentos passados são errados.
        """
        sys.stderr.write('Erro: %s\n' % msg)
        self.print_help()
        sys.exit(2)        
        
        
if __name__=='__main__':
    
    # Parse arguments from command line
    parser = MyParser()  
    args = parser.parse_args() 
    
    # Load csv file
    data = pd.read_csv(args.input_file, sep=args.sep, encoding=args.encoding)    
    
    univar.plot_dataframe_profile(data)
    
    #print data
    print args.input_file, args.sep, args.encoding
    


# -*- coding: utf-8 -*-
"""
Functions to create useful graphs for ploting distributions o single variables. Methods included:

 : plot_bar_chart
     Plot barplot showing the distribution of a categorical variable (for one or multiple series of data). 
 
 : plot_dataframe_profile
     Plot a grid of subplots in which each subplot shows the distribution of a variable of the data frame. For
     numerical variables a histogram is shown. For categorical, a barplot whose bars lengths are proportional
     to the percentage of each category in the column.
"""


import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import math

from matplotlib.gridspec import GridSpec


def _is_numeric(series):
    """ Determine if the pandas.core.series.Series is numeric """
    return series.dtype == 'int' or series.dtype == 'float'

def _is_categorical(series):
    """ Determine if the pandas.core.series.Series is categorical """
    return str(series.dtype) == 'object' or str(series.dtype) == 'category'

def _get_percentage_of_categorical(series):
    """ Return a Pandas series with the percents of each category, indexed by the categries names """
    counts = series.value_counts()
    porcentagens = 100 * (counts / sum(counts))
    return porcentagens


def add_extra_xaxis(fig, x, labels, padding=35):
    """
    Add a x axis bellow the figure (indeed bellow the ax returned by fig.gca()) having the labels
    in the x positions. The axis is added by first adding an entire new axes and the hiding all
    parts, except the xaxis.

     Parameters
    ------------

    fig : Figure
        The figure where to add the xaxis.

    x : list
        List of numbers specifying the x positions.

    labels : list
        List of strings specifying the labels to place in the x positions.

    padding : int
        How much space should be added between the figure and the new x axis bellow it.

     Returns
    ---------

    new_ax : Axes
        Return the axes added to the image (where the x axis was plotted).

    """

    # Add some space bellow the figure
    fig.subplots_adjust(bottom=0.2)

    # Get current ax
    ax = fig.gca()

    # Add a new ax to the figure
    new_ax = fig.add_axes(ax.get_position())

    # Hide the the plot area and the yaxis
    new_ax.patch.set_visible(False)
    new_ax.yaxis.set_visible(False)

    # Hide spines (unless the boottom one)
    for spinename, spine in new_ax.spines.iteritems():
        if spinename != 'bottom':
            spine.set_visible(False)

    # Set the ....
    new_ax.spines['bottom'].set_position(('outward', padding))

    # Change tick labels
    plt.xticks([0] + x, [''] + labels) # the [0] and [''] stuff is to add an empty lable in the first position

    return new_ax



def plot_bar_chart(data, cmap='Accent', color=None, xlabel='', ylabel='', title='', width=0.9, xticks_rotation=0, series_legends=None, ax=None):
    """
    Plot barplot showing the distribution of a categorical variable (for one or multiple series of data).

    Parameters
    ----------

    data : list or pandas.core.series.Series
        Pandas series (with categorical data) or list of series. For multiple series, each categories are grouped together.

    cmap : matplotlib.colors.LinearSegmentedColormap
        Colormap to use for determining each category color. Default is 'Accent'.

    color : color to use in the graph (overrides colormap colors). Default is None, which means not to override colormap colors.

    xlabel, ylabel, title : guess what...

    width : float
        Width of each bar. Default is 0.9

    xticks_rotation : float
        Degrees to rotate x labels.

    series_legends : list
        Legends to identify each series.

    legend : boolean
        Whether to show or not the legend box.

    ax : matplotlib.axes_subplots.AxesSubplot
        Axes in which to plot the graph

    Returns
    -------

    ax : matplotlib.axes_subplots.AxesSubplot
        Axes where the figure has been plotted.

    """

    # In the case a unique series is passed as parameters, wrap it in a list
    if not isinstance(data, list):
        data = [data]
        
    # Determine series type (pandas can represent categorical data with at least three different classes!)
    data_type = str(data[0].dtype)
  
    # If the Series is of "objects" (usually strings)
    if data_type == 'object':
        
        # Get list of different strings
        categorias = np.unique(data)
        
    # If it is a series create by pd.Series([...], dtype='category') or equivalent (e.g. pd.Series(pd.Categorical([...], [...])) )
    elif data_type == 'category':             
        
        # Get list of categories
        categorias = data[0].cat.categories # We assume here that the first series has all categories (BUG: it can be wrong!!!)
        
    else:
        raise Exception('Invalid data type %s' % data[0].dtype )
    
    # Number of different categories
    num_de_categorias = len(categorias)
    
    # Number of series
    num_de_series = len(data)
    
    # Colormap to use in the graph
    colors = plt.cm.get_cmap(cmap, num_de_series)     
    
    # Position of each category
    x_locations = np.arange(num_de_categorias) * num_de_series
    
    # Create figure
    if ax == None:
        fig, ax = plt.subplots()
    
    # Create series names, if not provided    
    if series_legends == None:    
        series_legends = ['Series %i' % i for i in range(1, num_de_series + 1)]    
    
    # For each series
    for idx, series in enumerate(data):
               
        # Compute the porcentage of each category
        porcentagens = _get_percentage_of_categorical(series)      
        porcentagens = porcentagens[categorias] # Assure that percentages will be returned in the same order as the legend
        
        # Determine color to use in this series
        series_color = color if color != None else colors(idx)      
        
        # Plot bars
        ax.bar(x_locations + idx*width, porcentagens, width, color=series_color, label=series_legends[idx])

    # Plot title and axis labels
    ax.set_ylabel(ylabel)
    ax.set_xlabel(xlabel)
    ax.set_title(title)    
            
    # If there is more than one series
    if num_de_series > 1:
        
        # Show legend
        ax.legend(loc='best')
        
        # Draw categories names
        #...        
        
    # If there is just one series
    else:
        
        # Draw categories names
        ax.set_xticks(x_locations + width/2)
        categorias = [cat.decode('utf8') for cat in categorias] # Convert categires names to utf8 (matplotlib doesn't non ascii chars)
        ax.set_xticklabels(categorias, rotation=xticks_rotation)
    
    
    # Add some small gap before the first and after the last category
    ax.set_xlim(min(x_locations) - width/9, max(x_locations) + width*num_de_series + width/9)

    # Return the axes in which the graph was plotted
    return ax


def plot_dataframe_profile(data_frame, include_cols=None, exclude_cols=None, shape = None, hspace=0.4, wspace=0.2, default_xticks_rotation=0, xticks_rotation={}, default_color='blue', colors={}, bins=30, title='', titles={}, default_ylabel='', ylabels={}, default_xlabel='', xlabels={}):
    """
    Plot a grid of subplots in which each subplot shows the distribution of a variable of the data frame. For
    numerical variabels a histogram is shown. For categorical, a barplot whose bars lengths are proportional
    to the percentage of each category in the column.
    
    The subplots are drawn in the default column order in the dataframe, or in the order specified in include_cols.
    
    Parameters
    ----------
    
    data_frame : pandas.Dataframe
        The Pandas data frame whose columns will be plotted.

    include_cols : list
        A list with the column names that should be included in the plot. Default is None, which means all columns.
        If this parameter is used, just the columns specified are included, in the given order.

    remove_cols : list
        A list with the column names that should not be included in the plot. Columns listed here will be removed
        even if included in the include_cols list.

    shape : tuple
        A (rows, columns) tuples indicating the shape of the grid. Default is to use the square root of the number of variables
        as the number of columns, and as many rows as necessary to show all variables.

    hspace : float
        Vertical space between subplots.

    wspace : float
        Horizontal space betwwn subplots.

    default_xticks_rotation : float
        Default roatation in degrees of the category names in bar grpahs.

    xticks_rotation : dict
        Dictionary associating column names to xtick label orientation (in degrees).

    default_color : str
        Default color name for all subplots.

    colors : dict
        Dictionary associating column names to colors (override defaults color).

    title : str
        General title to use above all subplots.

    titles : dict
        Dictionary associating column names to titles to use in each subplot. If a value is not specified for a
        column, the colum name is used as title.

    default_ylabel : str
        Default lable to use in the y axis.

    ylabels : dict
        Dictionary associating columnnames to y labels to use in each subplot. If a value is not specified for a
        column, the default_ylabel is used.

    default_xlabel : str
        Default lable to use in the y axis.

    xlabels : dict
        Dictionary associating columnnames to y labels to use in each subplot. If a value is not specified for a
        column, the default_ylabel is used.
                
                
    Returns
    -------
    
    gs : GridSpec
        Return the GridSpec created with the subplots.
    
    """
    
    # Keep just specified columns, if it is the case
    if include_cols != None:
        #data_frame = data_frame[include_cols]
        columns_kept = include_cols
    else:
        columns_kept = data_frame.columns
        
    # Remove cols specified
    if exclude_cols != None:
        columns_kept = [x for x in columns_kept if x not in exclude_cols]
            
    # Get number of columns
    num_of_columns = len(columns_kept)
    
    # Get the number of rows and cols
    if shape != None:
        rows, cols = shape  # Unpack tuple passed as parameter
    else:
        
        # Number of columns in the plot (as the square root (rounded) of the number of variables )
        cols = int(math.ceil(math.sqrt(num_of_columns)))             
        
        # Number or rows in the plot (the necessary to put all variables)
        rows = int(math.ceil(num_of_columns / float(cols)))
    
    # Create a grid with rows x cols subplots
    gs = GridSpec(rows, cols, hspace=hspace, wspace=wspace)        
    
    # For each column
    for col_idx, column_name in enumerate(columns_kept):        
        
        # Create the subplot
        ax = plt.subplot(gs[col_idx])        
        
        # Determine the rotation of xticks for this plot                
        rotation = xticks_rotation[column_name] if column_name in xticks_rotation else default_xticks_rotation            

        # Determine the color
        subplot_color = colors[column_name] if column_name in colors else default_color
        
        # Determine subplot title
        subplot_title = titles[column_name] if column_name in titles else column_name.decode('utf8')    
        
        # Is it is categorical
        if _is_categorical(data_frame[column_name]):                      
            
            # Plot barplot
            plot_bar_chart(data_frame[column_name], ax=ax, title=subplot_title, xticks_rotation=rotation, color=subplot_color)         
        
        # If it is numeric
        if _is_numeric(data_frame[column_name]):
            
            # Calculate the weiths used to transform data in percents
            weigths = (np.zeros_like(data_frame[column_name]) + 100.) / len(data_frame[column_name])
            
            # Plot histogram
            ax.hist(data_frame[column_name], weights=weigths, color=subplot_color, bins=bins)
            ax.set_title(subplot_title)
            ax.grid()
            
        # Set the y label
        ylabel = ylabels[column_name] if column_name in ylabels else default_ylabel
        ax.set_ylabel(ylabel)
        
        # Set the x Label
        xlabel = xlabels[column_name] if column_name in xlabels else default_xlabel
        ax.set_xlabel(xlabel)        
            
    # Set the general title
    plt.suptitle(title)
            
    # Return the GridSpec with subplots
    return gs

if __name__=='__main__':
    
    pass


# -*- coding: utf-8 -*-
"""
Functions to plot maps and coordinates.

TODO:
 - Replace box value given in aspect_ratio by a box parameter
 - make color attribute work with connect_points
 - Return ax



"""

from math import radians, cos, sin, asin, sqrt
from mpl_toolkits.basemap import Basemap
import matplotlib.pylab as plt

import numpy as np
import gc


def plot_path_from_above(lon, lat, padding=1., show_map=True, color='red', aspect_ratio=3./4, ax=None):
    """

    Parameters
    ----------
        lon : pd.Series
            Series containing the list of longitudes.

        lat : pd.Series
            Series containing the list of latitudes.

        padding : float
            Space to add arond path (in degrees). Default is 2 degrees.

        show_map : boolean
            Whether to draw the map around the path. Default is True.

        color : color or list of colors.
            The color of the path. Default is red. Optionally, if the parameter connect_poiints is False, a list of colors
            can be passed (having the same size as the number of dots). Example:
            >> colors = plt.cm.get_cmap('coolwarm', numbr_of_colors)
            >> color_idx = [ int(i) for i in color_values ]
            >> geo.plot_path_from_above(lon, lat, color=colors(color_idxs))

        aspect_ratio : float or str or tuple
            Aspect ration of the map. Default is 3/4. The values 'square' and 'fit_extremities' can also be specified. The first
            creates a square map. The second, adds just enough space to fit the extremities (still adding the padding). The corrdinates
            of the box containing the map can also be specified as (min_lat, max_lat, min_lon, max_lon).

        ax : AxesSubplot
            Axes where to plot the map.

    """

    # Determine min and max longitude and latitude and add padding around them
    min_lat = lat.min()
    max_lat = lat.max()
    min_lon = lon.min()
    max_lon = lon.max()


    # Determine the rightmost point of the graph
    if aspect_ratio=='fit_extremities':

        # The rightmost point is just the max lat added with the padding
        max_lat = max_lat # What's this?
        pass

    elif aspect_ratio=='square':

        # The rightmost point the the that make the map a squared box
        max_lat = min_lat + np.abs(max_lon - min_lon) * 3/4 # The 3/4 is need to take into account the different scales between longitudes and latitudes

    elif isinstance(aspect_ratio, float):

        # We reduce some space from the latitude dimension to make the map obey the aspect ration given in the parameter aspect_ratio
        max_lat = min_lat + np.abs(max_lon - min_lon) * 3/4
        min_lat = min_lat + np.abs(max_lat - min_lat) * (0.5 - aspect_ratio/2) # make the min_lat a little higher
        max_lat = max_lat - np.abs(max_lat - min_lat) * (0.5 - aspect_ratio/2) # make the max_lat a little lower

    elif len(aspect_ratio)==4:

        # Get the coordinates provided
        min_lat, max_lat, min_lon, max_lon = aspect_ratio

    else:
        raise Exception('Invalid value for parameter aspect_ratio: %s' % aspect_ratio)


    # Add some space around the path
    min_lat = min_lat - padding * 3/4 # The fator 3/4 is needed to take into account the different scales between longitudes and latitudes
    max_lat = max_lat + padding * 3/4
    min_lon = min_lon - padding
    max_lon = max_lon + padding




    # Draw map, if specified
    if show_map:

        # Create map centered around the path
        m = Basemap(projection='merc',
            resolution = 'l', area_thresh = 1,
            llcrnrlon= min_lon,
            urcrnrlon= max_lon,
            llcrnrlat= min_lat,
            urcrnrlat= max_lat,
            ax=ax
        )

        # Draw paralles and meridians
        m.drawparallels(np.arange(-80.,81.,2.))
        m.drawmeridians(np.arange(-180.,181.,2.))

        # Draw relief and rivers
        m.shadedrelief()
        m.drawrivers()

        # Draw coordinates on map
        m.scatter(lon.values, lat.values, latlon=True, color=color, ax=ax)
    else:

        # Draw coordinates without using a mp
        ax.scatter(lon.values, lat.values, color=color)

    gc.collect()


def haversine(lon1, lat1, lon2, lat2):
    """
    Calculate the great circle distance between two points 
    on the earth (specified in decimal degrees)
    
    [Function found on many different sites on internet]
    """
    # convert decimal degrees to radians 
    lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

    # haversine formula 
    dlon = lon2 - lon1 
    dlat = lat2 - lat1 
    a = sin(dlat/2)**2 + cos(lat1) * cos(lat2) * sin(dlon/2)**2
    c = 2 * asin(sqrt(a)) 
    r = 6371 # Radius of earth in kilometers. Use 3956 for miles
    return c * r

def _lineMagnitude (x1, y1, x2, y2):
    """
    Returns the euclidean distance between two points.
    """
    lineMagnitude = sqrt(pow((x2 - x1), 2)+ pow((y2 - y1), 2))
    return lineMagnitude


def distance_from_point_to_segment (px, py, x1, y1, x2, y2):
    """
    Returns the minimum distance between a point (px, py) and a line segment (x1, y1, x2, y2).

    Method adapted from Map Rantala, available at:
    https://nodedangles.wordpress.com/2010/05/16/measuring-distance-from-a-point-to-a-line-segment/
    """
    LineMag = _lineMagnitude(x1, y1, x2, y2)

    if LineMag < 0.00000001:
        DistancePointLine = 9999
        return DistancePointLine

    u1 = (((px - x1) * (x2 - x1)) + ((py - y1) * (y2 - y1)))
    u = u1 / (LineMag * LineMag)

    if (u < 0.00001) or (u > 1):
        #// closest point does not fall within the line segment, take the shorter distance
        #// to an endpoint
        ix = _lineMagnitude(px, py, x1, y1)
        iy = _lineMagnitude(px, py, x2, y2)
        if ix > iy:
            DistancePointLine = iy
        else:
            DistancePointLine = ix
    else:
        # Intersecting point is on the line, use the formula
        ix = x1 + u * (x2 - x1)
        iy = y1 + u * (y2 - y1)
        DistancePointLine = _lineMagnitude(px, py, ix, iy)

    return DistancePointLine



if __name__=='__main__':

    pass



# -*- coding: utf-8 -*-
"""
Extends matplotlib scatter plot to add tooltips that are shown when the mouse is over the point.

TODO: Instead of calling plot for each point, we should use directly plt.scatter function and then get the points and add the annotations.
"""

import matplotlib.pyplot as plt


def drawTipBox(x, y, text, color, ax, alpha=0.9,):
    """
    Adds a tootip box with text in he point on the ax specified with the color and alpha parameters.
    """

    # Containing box parameters
    bbox_props = dict(
        boxstyle="round,pad=0.80",
        fc=color,
        ec='none',
        alpha=alpha
    )

    # Arrow parameters (bellow the box)
    arrrow_props = dict(
        arrowstyle="wedge,tail_width=2.",
        fc=color,
        ec="none",
        patchA=None,
        relpos=(0.5, .5),
    )

    # Draw the box on the ax
    ann = ax.annotate(
        text,
        xy=(x, y),
        xycoords='data',
        xytext=(5, 35),
        textcoords='offset points',
        size=12,
        va="bottom",
        bbox = bbox_props,
        arrowprops = arrrow_props
    )

    # Return the annotation
    return ann



def plot_mouse_over_scatter(x, y, tips, box_color=(0.95, 0.90, 1), c=None, marker='o', ax=None, **kwargs):
    """
    Draw a scatter plot adding a tooltip to be show when the mouse is passes over the point.

     Parameters
    ------------
    TODO:


    """

    # Set the default color, if a color array is not given
    if c is None:
        c = len(x)*['blue']

    assert len(x) == len(y) == len(tips) == len(c)

    # Initialize list of points and annotations
    points_with_annotation = []

    # Get current ax
    if ax is None:
        ax = plt.gca()

    # For pair of point and tip
    for i, x, y, tip in zip(range(0, len(c)), x, y, tips):

        # Draw the point ( the comma after point "automatically extracts" the item from the list
        point, = ax.plot(x, y, marker, color=c[i], **kwargs)

        # Draw the annotation box
        annotation = drawTipBox(x, y, tip, box_color, ax=ax)

        # Disable the annotation visibility
        annotation.set_visible(False)

        # Store the point with its annotation in a list
        points_with_annotation.append([point, annotation])

    # Function to show/hide points with mouseover event (to be execute whenever the mouse is moved)
    def on_move(event):


        # For each point, annotation pair
        for point, annotation in points_with_annotation:

            # Determine whether it should be visible (when the space occupied by the point contains the coordinates of the mouseover event
            should_be_visible = (point.contains(event)[0] == True)

            # If the visibility of a point changed
            if should_be_visible != annotation.get_visible():

                # Show the point and redraw
                annotation.set_visible(should_be_visible)
                plt.draw()


    # Get current figure
    fig = plt.gcf()

    # Register the move event with the on_move function
    on_move_id = fig.canvas.mpl_connect('motion_notify_event', on_move)

    plt.show()




